@@ -14,6 +14,7 @@
 # MDAnalysis: A Python package for the rapid analysis of molecular dynamics
 # simulations. In S. Benthall and S. Rostrup editors, Proceedings of the 15th
 # Python in Science Conference, pages 102-109, Austin, TX, 2016. SciPy.
+# doi: 10.25080/majora-629e541a-00e
 #
 # N. Michaud-Agrawal, E. J. Denning, T. B. Woolf, and O. Beckstein.
 # MDAnalysis: A Toolkit for the Analysis of Molecular Dynamics Simulations.
@@ -1406,3 +1407,21 @@ class TestInitGroup(object):
 
         with pytest.raises(TypeError):
             cls(0, 2, 4)  # missing Universe
+
+
+class TestDecorator(object):
+    @groups.check_pbc_and_unwrap
+    def dummy_funtion(cls, compound="group", pbc=True, unwrap=True):
+        return 0
+
+    @pytest.mark.parametrize('compound', ('fragments', 'molecules', 'residues',
+                                          'group', 'segments'))
+    @pytest.mark.parametrize('pbc', (True, False))
+    @pytest.mark.parametrize('unwrap', (True, False))
+    def test_decorator(self, compound, pbc, unwrap):
+
+        if compound == 'group' and pbc and unwrap:
+            with pytest.raises(ValueError):
+                self.dummy_funtion(compound=compound, pbc=pbc, unwrap=unwrap)
+        else:
+            assert_equal(self.dummy_funtion(compound=compound, pbc=pbc, unwrap=unwrap), 0)
\ No newline at end of file
